@@ -2,22 +2,22 @@ package metrics
 
 import (
 	"context"
+	"strconv"
 	"time"
 
 	"github.com/prometheus/client_golang/prometheus"
 	log "github.com/sirupsen/logrus"
-
 	batchv1 "k8s.io/api/batch/v1"
-	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/wait"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/manager"
 	"sigs.k8s.io/controller-runtime/pkg/metrics"
 
-	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
-	hiveintv1alpha1 "github.com/openshift/hive/pkg/apis/hiveinternal/v1alpha1"
+	hivev1 "github.com/openshift/hive/apis/hive/v1"
+	hiveintv1alpha1 "github.com/openshift/hive/apis/hiveinternal/v1alpha1"
 	"github.com/openshift/hive/pkg/constants"
+	controllerutils "github.com/openshift/hive/pkg/controller/utils"
 	"github.com/openshift/hive/pkg/imageset"
 )
 
@@ -46,7 +46,7 @@ var (
 	}, []string{"cluster_type", "age_lt", "deprovisioning_gt"})
 	metricClusterDeploymentsWithConditionTotal = prometheus.NewGaugeVec(prometheus.GaugeOpts{
 		Name: "hive_cluster_deployments_conditions",
-		Help: "Total number of cluster deployments by type with conditions.",
+		Help: "Total number of cluster deployments by type with conditions in their undesired state",
 	}, []string{"cluster_type", "age_lt", "condition"})
 	metricInstallJobsTotal = prometheus.NewGaugeVec(prometheus.GaugeOpts{
 		Name: "hive_install_jobs",
@@ -98,6 +98,17 @@ var (
 		},
 		[]string{"controller", "outcome"},
 	)
+	// metricClusterDeploymentSyncsetPaused tracks ClusterDeployments which
+	// are NOT being synchronized by Hive.  This is achieved by the presence
+	// of a hive.openshift.io/syncset-pause="true" annotation.  The value is
+	// a boolean with "1" indicating the annotation is present.
+	metricClusterDeploymentSyncsetPaused = prometheus.NewGaugeVec(
+		prometheus.GaugeOpts{
+			Name: "hive_cluster_deployment_syncset_paused",
+			Help: "Whether Hive has paused syncing to the cluster",
+		},
+		[]string{"cluster_deployment", "namespace", "cluster_type"},
+	)
 )
 
 // ReconcileOutcome is used in controller "reconcile complete" log entries, and the metricControllerReconcileTime
@@ -130,6 +141,7 @@ func init() {
 	metrics.Registry.MustRegister(metricControllerReconcileTime)
 
 	metrics.Registry.MustRegister(MetricClusterDeploymentDeprovisioningUnderwaySeconds)
+	metrics.Registry.MustRegister(metricClusterDeploymentSyncsetPaused)
 }
 
 // Add creates a new metrics Calculator and adds it to the Manager.
@@ -138,7 +150,8 @@ func Add(mgr manager.Manager) error {
 		Client:   mgr.GetClient(),
 		Interval: 2 * time.Minute,
 	}
-	metrics.Registry.MustRegister(newProvisioningUnderwayCollector(mgr.GetClient()))
+	metrics.Registry.MustRegister(newProvisioningUnderwaySecondsCollector(mgr.GetClient(), 1*time.Hour))
+	metrics.Registry.MustRegister(newProvisioningUnderwayInstallRestartsCollector(mgr.GetClient(), 1))
 	err := mgr.Add(mc)
 	if err != nil {
 		return err
@@ -160,11 +173,11 @@ type Calculator struct {
 }
 
 // Start begins the metrics calculation loop.
-func (mc *Calculator) Start(stopCh <-chan struct{}) error {
+func (mc *Calculator) Start(ctx context.Context) error {
 	log.Info("started metrics calculator goroutine")
 
 	// Run forever, sleep at the end:
-	wait.Until(func() {
+	wait.UntilWithContext(ctx, func(ctx context.Context) {
 		mcLog := log.WithField("controller", "metrics")
 		recobsrv := NewReconcileObserver(ControllerName, mcLog)
 		defer recobsrv.ObserveControllerReconcileTime()
@@ -172,7 +185,7 @@ func (mc *Calculator) Start(stopCh <-chan struct{}) error {
 		mcLog.Info("calculating metrics across all ClusterDeployments")
 		// Load all ClusterDeployments so we can accumulate facts about them.
 		clusterDeployments := &hivev1.ClusterDeploymentList{}
-		err := mc.Client.List(context.Background(), clusterDeployments)
+		err := mc.Client.List(ctx, clusterDeployments)
 		if err != nil {
 			log.WithError(err).Error("error listing cluster deployments")
 		} else {
@@ -182,6 +195,7 @@ func (mc *Calculator) Start(stopCh <-chan struct{}) error {
 				return
 			}
 			for _, cd := range clusterDeployments.Items {
+				clusterType := GetClusterDeploymentType(&cd)
 				accumulator.processCluster(&cd)
 
 				if cd.DeletionTimestamp != nil {
@@ -192,9 +206,25 @@ func (mc *Calculator) Start(stopCh <-chan struct{}) error {
 					MetricClusterDeploymentDeprovisioningUnderwaySeconds.WithLabelValues(
 						cd.Name,
 						cd.Namespace,
-						GetClusterDeploymentType(&cd)).Set(
+						clusterType).Set(
 						time.Since(cd.CreationTimestamp.Time).Seconds())
 				}
+
+				if paused, err := strconv.ParseBool(cd.Annotations[constants.SyncsetPauseAnnotation]); err == nil && paused {
+					metricClusterDeploymentSyncsetPaused.WithLabelValues(
+						cd.Name,
+						cd.Namespace,
+						clusterType).Set(1.0)
+				} else {
+					cleared := metricClusterDeploymentSyncsetPaused.Delete(map[string]string{
+						"cluster_deployment": cd.Name,
+						"namespace":          cd.Namespace,
+						"cluster_type":       clusterType,
+					})
+					if cleared {
+						mcLog.Infof("cleared metric: %v", metricClusterDeploymentSyncsetPaused)
+					}
+				}
 			}
 
 			accumulator.setMetrics(metricClusterDeploymentsTotal,
@@ -226,7 +256,7 @@ func (mc *Calculator) Start(stopCh <-chan struct{}) error {
 		// install job metrics
 		installJobs := &batchv1.JobList{}
 		installJobLabelSelector := map[string]string{constants.InstallJobLabel: "true"}
-		err = mc.Client.List(context.Background(), installJobs, client.MatchingLabels(installJobLabelSelector))
+		err = mc.Client.List(ctx, installJobs, client.MatchingLabels(installJobLabelSelector))
 		if err != nil {
 			log.WithError(err).Error("error listing install jobs")
 		} else {
@@ -246,7 +276,7 @@ func (mc *Calculator) Start(stopCh <-chan struct{}) error {
 		// uninstall job metrics
 		uninstallJobs := &batchv1.JobList{}
 		uninstallJobLabelSelector := map[string]string{constants.UninstallJobLabel: "true"}
-		err = mc.Client.List(context.Background(), uninstallJobs, client.MatchingLabels(uninstallJobLabelSelector))
+		err = mc.Client.List(ctx, uninstallJobs, client.MatchingLabels(uninstallJobLabelSelector))
 		if err != nil {
 			log.WithError(err).Error("error listing uninstall jobs")
 		} else {
@@ -266,7 +296,7 @@ func (mc *Calculator) Start(stopCh <-chan struct{}) error {
 		// imageset job metrics
 		imagesetJobs := &batchv1.JobList{}
 		imagesetJobLabelSelector := map[string]string{imageset.ImagesetJobLabel: "true"}
-		err = mc.Client.List(context.Background(), imagesetJobs, client.MatchingLabels(imagesetJobLabelSelector))
+		err = mc.Client.List(ctx, imagesetJobs, client.MatchingLabels(imagesetJobLabelSelector))
 		if err != nil {
 			log.WithError(err).Error("error listing imageset jobs")
 		} else {
@@ -283,7 +313,7 @@ func (mc *Calculator) Start(stopCh <-chan struct{}) error {
 		}
 
 		mc.calculateSelectorSyncSetMetrics(mcLog)
-	}, mc.Interval, stopCh)
+	}, mc.Interval)
 
 	return nil
 }
@@ -424,9 +454,6 @@ func newClusterAccumulator(ageFilter string, durationBuckets []string) (*cluster
 		ca.deprovisioning[durStr] = map[string]int{}
 	}
 
-	for _, cdct := range hivev1.AllClusterDeploymentConditions {
-		ca.conditions[cdct] = map[string]int{}
-	}
 	return ca, nil
 }
 
@@ -507,15 +534,8 @@ func (ca *clusterAccumulator) processCluster(cd *hivev1.ClusterDeployment) {
 
 	// Process conditions regardless if installed or not:
 	for _, cond := range cd.Status.Conditions {
-		if cond.Status == corev1.ConditionTrue {
-			// Should have been handled by the initialization above which ensures we have 0's in the metrics for
-			// conditions that are not currently present on any clusters. This is a safety check to avoid crashing Hive
-			// in the event a developer adds a new condition but misses the list of all types.
-			if ca.conditions[cond.Type] == nil {
-				log.Warnf("condition type %s missing from AllClusterDeploymentConditions slice", cond.Type)
-				ca.conditions[cond.Type] = map[string]int{}
-			}
-			ca.conditions[cond.Type][clusterType]++
+		if !controllerutils.IsConditionInDesiredState(cond) {
+			ca.addConditionToMap(cond.Type, clusterType)
 		}
 	}
 }
@@ -607,4 +627,11 @@ func (ro *ReconcileObserver) SetOutcome(outcome ReconcileOutcome) {
 	ro.outcome = outcome
 }
 
+func (ca *clusterAccumulator) addConditionToMap(cond hivev1.ClusterDeploymentConditionType, clusterType string) {
+	if ca.conditions[cond] == nil {
+		ca.conditions[cond] = map[string]int{}
+	}
+	ca.conditions[cond][clusterType]++
+}
+
 var elapsedDurationBuckets = []time.Duration{2 * time.Minute, time.Minute, 30 * time.Second, 10 * time.Second, 5 * time.Second, time.Second, 0}
